package Time::Available;

use 5.001;
use strict;
use warnings;

require Exporter;

our @ISA = qw(Exporter);

our %EXPORT_TAGS = (
	'days' => [ qw(
		DAY_MONDAY
		DAY_TUESDAY
		DAY_WEDNESDAY
		DAY_THURSDAY
		DAY_FRIDAY
		DAY_SATURDAY
		DAY_SUNDAY
		DAY_WEEKDAY
		DAY_WEEKEND
		DAY_EVERYDAY
	) ],
	'fmt_interval' => [ qw(fmt_interval) ]
);

our @EXPORT_OK = (
	@{ $EXPORT_TAGS{'days'} },
	@{ $EXPORT_TAGS{'fmt_interval'} }
	);

our @EXPORT;	# don't export anything by default!

our $VERSION = '0.01';

# define some constants used later
use constant DAY_MONDAY    => 0x01;
use constant DAY_TUESDAY   => 0x02;
use constant DAY_WEDNESDAY => 0x04;
use constant DAY_THURSDAY  => 0x08;
use constant DAY_FRIDAY    => 0x10;
use constant DAY_SATURDAY  => 0x20;
use constant DAY_SUNDAY    => 0x40;
use constant DAY_WEEKDAY   => 0x1F;
use constant DAY_WEEKEND   => 0x60;
use constant DAY_EVERYDAY  => 0x7F;

use constant SEC_PER_DAY   => 86400;

my $debug = 0;

#
# make new instance
#
sub new {
	my $class = shift;
	my $self = {};
	bless($self, $class);
	$self->{ARGS} = {@_};
	$debug = $self->{ARGS}->{DEBUG};

	die("need start time") if (! $self->{ARGS}->{start});

	# calc start and stop seconds
	my ($hh,$mm,$ss) = split(/:/,$self->{ARGS}->{start},3);
	my $s = $hh * 3600 || die("need at least hour specified for start time");
	$s += $mm * 60 if ($mm);
	$s += $ss if ($ss);
	$self->{start} = $s;

	die("need end time") if (! $self->{ARGS}->{end});

	($hh,$mm,$ss) = split(/:/,$self->{ARGS}->{end},3);
	$s = $hh * 3600 || die("need at least hour specified for end time");
	$s += $mm * 60 if ($mm);
	$self->{end} = $s;

	die("need dayMask specified") if (! $self->{ARGS}->{dayMask});

	$self->{dayMask} = $self->{ARGS}->{dayMask};

	$self ? return $self : return undef;
}



#
# this sub (originally from Time::Avail) will return if day is applicable
#

sub _dayOk($) {
	my $self = shift;
	my $day = shift || return;

	my $dayMask = $self->{dayMask};

	my $dayOk = 0;

	if( ( $day == 0 ) && ( $dayMask & DAY_SUNDAY ) ) {
		$dayOk = 1;
	} elsif( ( $day == 1) && ( $dayMask & DAY_MONDAY ) ) {
		$dayOk = 1;
	} elsif( ($day == 2) && ( $dayMask & DAY_TUESDAY ) ) {
		$dayOk = 1;
	} elsif( ($day == 3)  && ( $dayMask & DAY_WEDNESDAY ) ) {
		$dayOk = 1;
	} elsif( ( $day == 4) && ( $dayMask & DAY_THURSDAY ) ) {
		$dayOk = 1;
	} elsif( ( $day == 5 ) && ( $dayMask & DAY_FRIDAY ) ) {
		$dayOk = 1;
	} elsif( ( $day == 6 ) && ( $dayMask & DAY_SATURDAY ) ) {
		$dayOk = 1;
	}

	print STDERR "day: $day dayMask: ",unpack("B32", pack("N", $dayMask))," ok: $dayOk\n" if ($debug);

	return $dayOk;
}


#
# this will return number of seconds that service is available if passed
# uptime of service
#

sub uptime {
	my $self = shift;

	my $time = shift || die "need uptime timestamp to calcualte uptime";

	# calculate offset -- that is number of seconds since midnight
	my @lt = localtime($time);
	my $offset = $lt[2];	# hour
	$offset *= 60;		# convert to minutes
	$offset += $lt[1];	# minutes
	$offset *= 60;		# convert to seconds
	$offset += $lt[0];

	# check if day falls into dayMask
	return 0 if (! $self->_dayOk($lt[6]) );

	my $s=0;

	my $start = $self->{start};
	my $end = $self->{end};

	print STDERR "start: $start end: $end time: $offset\n" if ($debug);

	if ( $end > $start ) {
		if ($offset < $start) {
			$s = $end - $start;
		} elsif ($offset < $end) {
			$s = $end - $offset;
		}
	} elsif ( $start > $end ) {	# over midnight
		if ( $offset < $end ) {
			if ( $offset < $start) {
				$s = SEC_PER_DAY - $start + $end - $offset;
			} else {
				$s = SEC_PER_DAY - $start + $end;
			}
		} else {
			if ( $offset < $start ) {
				$s = SEC_PER_DAY - $start;
			} else {
				$s = SEC_PER_DAY - $offset;
			}
		}
	}
		
	return $s;
}

#
# this auxillary function will pretty-format interval in [days]d hh:mm:ss
#

sub fmt_interval {
	my $s = shift || 0;
	my $out = "";

	my $d = int($s/(24*60*60));
	$s = $s % (24*60*60);
	my $h = int($s/(60*60));
	$s = $s % (60*60);
	my $m = int($s/60);
	$s = $s % 60;
	
	$out .= $d."d " if ($d > 0);

	$out .= sprintf("%02d:%02d:%02d",$h,$m,$s);

	return $out;
}

1;
__END__

=head1 NAME

Time::Available - Perl extension to calculate time availability

=head1 SYNOPSIS

  use Time::Available;

  # init interval and dayMask
  my $interval = new( start=>'07:00', stop=>'17:00',
  	dayMask=> Time::Available::DAY_WEEKDAY );

  # alternative way to init module using exporting of days
  use Time::Available qw(:days);
  my $interval = new( start=>'07:00', stop=>'17:00',
  	dayMask=> DAY_WEEKDAY );

  # calculate current availability in seconds
  print $interval->uptime(localtime);

  # calculate availablity in seconds from interval of uptime
  print $interval->interval($utime1,$utime2);

  # pretty print interval data (this will produce output '1d 11:11:11')
  use Time::Available qw(:fmt_interval);
  print fmt_interval(126671);

=head1 DESCRIPTION

Time::Available is used to calculate availability of some resource if start
end end time of availability is available. That availability is calculated
relative to some interval which is defined when new instance of module is
created.

Start and end dates must be specified in 24-hour format. You can specify
just hour, hour:minute or hour:minute:seconds format.

The B<dayMask> parameter is constructed by OR'ing together one or more of
the following dayMask constants:

=over 4

=item *
Time::Avail::DAY_MONDAY

=item *
Time::Avail::DAY_TUESDAY

=item *
Time::Avail::DAY_WEDNESDAY

=item *
Time::Avail::DAY_THURSDAY

=item *
Time::Avail::DAY_FRIDAY

=item *
Time::Avail::DAY_SATURDAY

=item *
Time::Avail::DAY_SUNDAY

=item *
Time::Avail::DAY_WEEKDAY

=item *
Time::Avail::DAY_WEEKEND

=item *
Time::Avail::DAY_EVERYDAY

=back

FIXME

=head2 EXPORT

None by default.

If you specify B<:days>, Time::Available will export all
DAY_* constraints to your enviroment (causing possible pollution of name
space). You have been warned.

With B<:fmt_interval> it will include function B<fmt_interval> which will
pretty-format interval into [days]d hh:mm:ss.


=head1 HISTORY

=over 8

=item 0.01

Original version; based somewhat on Time::Avail code

=back

=head1 BUGS

=over 8

=item *
Use croak and not die in module for better error handling

=item *
Allow arbitary (array?) of holidays to be included.

=back

=head1 SEE ALSO

Time::Avail is CPAN module that started it all. However, it lacked
calculating of availability of some interval and precision in seconds, so
this module was born.

More information about this module might be found on
http://www.rot13.org/~dpavlin/projects.html#cpan

=head1 AUTHOR

Dobrica Pavlinusic, E<lt>dpavlin@rot13.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2003 by Dobrica Pavlinusic

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut


1;
